@@ -18,6 +18,7 @@ package org.gradle.internal.buildoption;
 
 import org.gradle.internal.service.scopes.Scope;
 import org.gradle.internal.service.scopes.ServiceScope;
+import org.jspecify.annotations.Nullable;
 
 /**
  * A service that determines the value for an {@link InternalOption}.
@@ -27,5 +28,5 @@ public interface InternalOptions {
     /**
      * Lookup the value for an {@link InternalOption}.
      */
-    <T> Option.Value<T> getOption(InternalOption<T> option);
+    <T extends @Nullable Object> Option.Value<T> getOption(InternalOption<T> option);
 }
