@@ -104,7 +104,6 @@ import org.gradle.internal.resource.TextUriResourceLoader;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
 import org.gradle.internal.typeconversion.TypeConverter;
-import org.gradle.invocation.GradleLifecycleActionExecutor;
 import org.gradle.listener.ClosureBackedMethodInvocationDispatch;
 import org.gradle.model.Model;
 import org.gradle.model.RuleSource;
@@ -123,9 +122,9 @@ import org.gradle.model.internal.registry.ModelRegistry;
 import org.gradle.model.internal.type.ModelType;
 import org.gradle.normalization.InputNormalizationHandler;
 import org.gradle.normalization.internal.InputNormalizationHandlerInternal;
-import org.gradle.plugin.software.internal.SoftwareFeatureApplicator;
-import org.gradle.plugin.software.internal.SoftwareFeatureRegistry;
-import org.gradle.plugin.software.internal.SoftwareFeatureSupportInternal;
+import org.gradle.plugin.software.internal.ProjectFeatureApplicator;
+import org.gradle.plugin.software.internal.ProjectFeatureDeclarations;
+import org.gradle.plugin.software.internal.ProjectFeatureSupportInternal;
 import org.gradle.util.Configurable;
 import org.gradle.util.Path;
 import org.gradle.util.internal.ClosureBackedAction;
@@ -194,8 +193,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     private AntBuilder ant;
 
-    private final int depth;
-
     private final TaskContainerInternal taskContainer;
 
     private ListenerBroadcast<ProjectEvaluationListener> evaluationListener = newProjectEvaluationListenerBroadcast();
@@ -210,8 +207,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     private boolean preparedForRuleBasedPlugins;
 
-    private final GradleLifecycleActionExecutor gradleLifecycleActionExecutor;
-
     @Nullable
     private Object beforeProjectActionState;
 
@@ -239,23 +234,9 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
         this.buildScriptSource = buildScriptSource;
         this.gradle = gradle;
 
-        if (parent == null) {
-            depth = 0;
-        } else {
-            depth = parent.getDepth() + 1;
-        }
-
         services = serviceRegistryFactory.createFor(this);
         taskContainer = services.get(TaskContainerInternal.class);
-        gradleLifecycleActionExecutor = services.get(GradleLifecycleActionExecutor.class);
-        extensibleDynamicObject = new ExtensibleDynamicObject(
-            this,
-            new MutableStateAccessAwareDynamicObject(
-                new BeanDynamicObject(this, Project.class),
-                () -> gradleLifecycleActionExecutor.executeBeforeProjectFor(DefaultProject.this)
-            ),
-            services.get(InstantiatorFactory.class).decorateLenient(services)
-        );
+        extensibleDynamicObject = new ExtensibleDynamicObject(this, Project.class, services.get(InstantiatorFactory.class).decorateLenient(services));
 
         @Nullable DynamicObject parentInherited = services.get(CrossProjectModelAccess.class).parentProjectDynamicInheritedScope(this);
         if (parentInherited != null) {
@@ -263,7 +244,7 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
         }
         extensibleDynamicObject.addObject(taskContainer.getTasksAsDynamicObject(), ExtensibleDynamicObject.Location.AfterConvention);
 
-        SoftwareFeatureSupportInternal.attachLegacyDefinitionContext(this, services.get(SoftwareFeatureApplicator.class), services.get(SoftwareFeatureRegistry.class), getObjects());
+        ProjectFeatureSupportInternal.attachLegacyDefinitionContext(this, services.get(ProjectFeatureApplicator.class), services.get(ProjectFeatureDeclarations.class), getObjects());
 
         evaluationListener.add(gradle.getProjectEvaluationBroadcaster());
 
@@ -341,10 +322,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
         }
     }
 
-    private void onMutableStateAccess() {
-        gradleLifecycleActionExecutor.executeBeforeProjectFor(this);
-    }
-
     private ListenerBroadcast<ProjectEvaluationListener> newProjectEvaluationListenerBroadcast() {
         return new ListenerBroadcast<>(ProjectEvaluationListener.class);
     }
@@ -401,7 +378,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract ScriptHandlerInternal getBuildscript();
 
     @Override
@@ -411,7 +387,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public void setScript(groovy.lang.Script buildScript) {
-        onMutableStateAccess();
         extensibleDynamicObject.addObject(new BeanDynamicObject(buildScript).withNoProperties().withNotImplementsMissing(),
             ExtensibleDynamicObject.Location.BeforeConvention);
     }
@@ -465,19 +440,16 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
     @Override
     @Nullable
     public String getDescription() {
-        onMutableStateAccess();
         return description;
     }
 
     @Override
     public void setDescription(@Nullable String description) {
-        onMutableStateAccess();
         this.description = description;
     }
 
     @Override
     public Object getGroup() {
-        onMutableStateAccess();
         if (group == null) {
             group = getDefaultGroup();
         }
@@ -504,37 +476,31 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public void setGroup(Object group) {
-        onMutableStateAccess();
         this.group = group;
     }
 
     @Override
     public Object getVersion() {
-        onMutableStateAccess();
         return version == null ? DEFAULT_VERSION : version;
     }
 
     @Override
     public void setVersion(Object version) {
-        onMutableStateAccess();
         this.version = version;
     }
 
     @Override
     public Object getStatus() {
-        // onMutableStateAccess() triggered by #getInternalStatus()
         return getInternalStatus().get();
     }
 
     @Override
     public void setStatus(Object s) {
-        // onMutableStateAccess() triggered by #getInternalStatus()
         getInternalStatus().set(s);
     }
 
     @Override
     public Property<Object> getInternalStatus() {
-        onMutableStateAccess();
         if (status == null) {
             status = getObjects().property(Object.class).convention(DEFAULT_STATUS);
         }
@@ -553,19 +519,16 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public List<String> getDefaultTasks() {
-        onMutableStateAccess();
         return defaultTasks;
     }
 
     @Override
     public void setDefaultTasks(List<String> defaultTasks) {
-        onMutableStateAccess();
         this.defaultTasks = defaultTasks;
     }
 
     @Override
     public ProjectStateInternal getState() {
-        onMutableStateAccess();
         return state;
     }
 
@@ -583,17 +546,14 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract ArtifactHandler getArtifacts();
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract RepositoryHandler getRepositories();
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract RoleBasedConfigurationContainerInternal getConfigurations();
 
     @Override
@@ -624,7 +584,7 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public int getDepth() {
-        return depth;
+        return owner.getDepth();
     }
 
     @Inject
@@ -677,13 +637,11 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public ModelContainer<ProjectInternal> getModel() {
-        onMutableStateAccess();
         return getOwner();
     }
 
     @Override
     public PluginContainer getPlugins() {
-        onMutableStateAccess();
         return super.getPlugins();
     }
 
@@ -811,7 +769,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public AntBuilder createAntBuilder() {
-        onMutableStateAccess();
         return getAntBuilderFactory().createAntBuilder();
     }
 
@@ -830,7 +787,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public DefaultProject evaluate() {
-        // onMutableStateAccess() triggered by #getProjectEvaluator()
         getProjectEvaluator().evaluate(this, state);
         return this;
     }
@@ -843,7 +799,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public ProjectInternal bindAllModelRules() {
-        // onMutableStateAccess() triggered by #getModelRegistry
         try {
             getModelRegistry().bindAllReferences();
         } catch (Exception e) {
@@ -854,7 +809,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public TaskContainerInternal getTasks() {
-        onMutableStateAccess();
         return taskContainer;
     }
 
@@ -863,7 +817,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
         if (defaultTasks == null) {
             throw new InvalidUserDataException("Default tasks must not be null!");
         }
-        onMutableStateAccess();
         this.defaultTasks = new ArrayList<String>();
         for (String defaultTask : defaultTasks) {
             if (defaultTask == null) {
@@ -881,14 +834,12 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
     @Override
     @Deprecated
     public File getBuildDir() {
-        // onMutableStateAccess(); triggered by #getLayout()
         return getLayout().getBuildDirectory().getAsFile().get();
     }
 
     @Override
     @Deprecated
     public void setBuildDir(File path) {
-        // onMutableStateAccess(); triggered by #getLayout() in #setBuildDir(Object) below
         setBuildDir((Object) path);
     }
 
@@ -900,7 +851,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public void evaluationDependsOnChildren() {
-        onMutableStateAccess();
         for (ProjectState project : owner.getChildProjects()) {
             ProjectInternal defaultProjectToEvaluate = project.getMutableModel();
             evaluationDependsOn(defaultProjectToEvaluate);
@@ -912,7 +862,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
         if (isNullOrEmpty(path)) {
             throw new InvalidUserDataException("You must specify a project!");
         }
-        // onMutableStateAccess(); triggered by `evaluationDependsOn(ProjectInternal)`
         ProjectInternal projectToEvaluate = project(path);
         return evaluationDependsOn(projectToEvaluate);
     }
@@ -922,11 +871,19 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
             throw new CircularReferenceException(String.format("Circular referencing during evaluation for %s.",
                 projectToEvaluate));
         }
-        onMutableStateAccess();
         projectToEvaluate.getOwner().ensureConfigured();
         return projectToEvaluate;
     }
 
+    /**
+     * Returns the display name of this project in a human-readable format.
+     * <p>
+     * Currently:
+     * <ul>
+     *     <li>For the root project: {@code root project 'projectName'}</li>
+     *     <li>For subprojects: {@code project ':identity:path:of:project'}</li>
+     * </ul>
+     */
     @Override
     public String getDisplayName() {
         return owner.getDisplayName().getDisplayName();
@@ -939,7 +896,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public Map<Project, Set<Task>> getAllTasks(boolean recursive) {
-        onMutableStateAccess();
         final Map<Project, Set<Task>> foundTargets = new TreeMap<Project, Set<Task>>();
         Action<Project> action = new Action<Project>() {
             @Override
@@ -963,7 +919,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
         if (isNullOrEmpty(name)) {
             throw new InvalidUserDataException("Name is not specified!");
         }
-        onMutableStateAccess();
         final Set<Task> foundTasks = new HashSet<Task>();
         Action<Project> action = new Action<Project>() {
             @Override
@@ -999,7 +954,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract ProjectLayout getLayout();
 
     @Override
@@ -1105,7 +1059,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract DependencyHandler getDependencies();
 
     @Inject
@@ -1119,14 +1072,12 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public void beforeEvaluate(Action<? super Project> action) {
-        onMutableStateAccess();
         assertEagerContext("beforeEvaluate(Action)");
         evaluationListener.add("beforeEvaluate", getListenerBuildOperationDecorator().decorate("Project.beforeEvaluate", action));
     }
 
     @Override
     public void afterEvaluate(Action<? super Project> action) {
-        onMutableStateAccess();
         assertEagerContext("afterEvaluate(Action)");
         failAfterProjectIsEvaluated("afterEvaluate(Action)");
         evaluationListener.add("afterEvaluate", getListenerBuildOperationDecorator().decorate("Project.afterEvaluate", action));
@@ -1134,14 +1085,12 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public void beforeEvaluate(Closure closure) {
-        onMutableStateAccess();
         assertEagerContext("beforeEvaluate(Closure)");
         evaluationListener.add(new ClosureBackedMethodInvocationDispatch("beforeEvaluate", getListenerBuildOperationDecorator().decorate("Project.beforeEvaluate", Cast.<Closure<?>>uncheckedNonnullCast(closure))));
     }
 
     @Override
     public void afterEvaluate(Closure closure) {
-        onMutableStateAccess();
         assertEagerContext("afterEvaluate(Closure)");
         failAfterProjectIsEvaluated("afterEvaluate(Closure)");
         evaluationListener.add(new ClosureBackedMethodInvocationDispatch("afterEvaluate", getListenerBuildOperationDecorator().decorate("Project.afterEvaluate", Cast.<Closure<?>>uncheckedNonnullCast(closure))));
@@ -1169,7 +1118,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract SoftwareComponentContainer getComponents();
 
     @Override
@@ -1269,25 +1217,16 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public ServiceRegistry getServices() {
-        onMutableStateAccess();
         return services;
     }
 
-    @Override
-    public ServiceRegistryFactory getServiceRegistryFactory() {
-        onMutableStateAccess();
-        return services.get(ServiceRegistryFactory.class);
-    }
-
     @Override
     public AntBuilder ant(Closure configureClosure) {
-        // onMutableStateAccess() triggered by #getAnt
         return ConfigureUtil.configure(configureClosure, getAnt());
     }
 
     @Override
     public AntBuilder ant(Action<? super AntBuilder> configureAction) {
-        // onMutableStateAccess() triggered by #getAnt
         AntBuilder ant = getAnt();
         configureAction.execute(ant);
         return ant;
@@ -1325,104 +1264,87 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public void configurations(Closure configureClosure) {
-        // onMutableStateAccess() triggered by #getConfigurations()
         ((Configurable<?>) getConfigurations()).configure(configureClosure);
     }
 
     @Override
     public void repositories(Closure configureClosure) {
-        // onMutableStateAccess() triggered by #getRepositories()
         ConfigureUtil.configure(configureClosure, getRepositories());
     }
 
     @Override
     public void dependencies(Closure configureClosure) {
-        // onMutableStateAccess() triggered by #getDependencies()
         ConfigureUtil.configure(configureClosure, getDependencies());
     }
 
     @Override
     public void artifacts(Closure configureClosure) {
-        // onMutableStateAccess() triggered by #getArtifacts()
         ConfigureUtil.configure(configureClosure, getArtifacts());
     }
 
     @Override
     public void artifacts(Action<? super ArtifactHandler> configureAction) {
-        // onMutableStateAccess() triggered by #getArtifacts()
         configureAction.execute(getArtifacts());
     }
 
     @Override
     public void buildscript(Closure configureClosure) {
-        // onMutableStateAccess() triggered by #getBuildscript()
         ConfigureUtil.configure(configureClosure, getBuildscript());
     }
 
     @SuppressWarnings("deprecation")
     @Override
     public Task task(String task) {
-        onMutableStateAccess();
         return taskContainer.create(task);
     }
 
     @SuppressWarnings("deprecation")
     public Task task(Object task) {
-        onMutableStateAccess();
         return taskContainer.create(task.toString());
     }
 
     @SuppressWarnings("deprecation")
     @Override
     public Task task(String task, Action<? super Task> configureAction) {
-        onMutableStateAccess();
         return taskContainer.create(task, configureAction);
     }
 
     @SuppressWarnings("deprecation")
     @Override
     public Task task(String task, Closure configureClosure) {
-        onMutableStateAccess();
         return taskContainer.create(task).configure(configureClosure);
     }
 
     public Task task(Object task, Closure configureClosure) {
-        // onMutableStateAccess() triggered by #task(String, Closure)
         return task(task.toString(), configureClosure);
     }
 
     @SuppressWarnings("deprecation")
     @Override
     public Task task(Map options, String task) {
-        onMutableStateAccess();
         return taskContainer.create(addMaps(Cast.uncheckedNonnullCast(options), singletonMap(Task.TASK_NAME, task)));
     }
 
     public Task task(Map options, Object task) {
-        // onMutableStateAccess() triggered by #task(Map, String)
         return task(options, task.toString());
     }
 
     @SuppressWarnings("deprecation")
     @Override
     public Task task(Map options, String task, Closure configureClosure) {
-        onMutableStateAccess();
         return taskContainer.create(addMaps(Cast.uncheckedNonnullCast(options), singletonMap(Task.TASK_NAME, task))).configure(configureClosure);
     }
 
     public Task task(Map options, Object task, Closure configureClosure) {
-        // onMutableStateAccess() triggered by #task(Map, String, Closure)
         return task(options, task.toString(), configureClosure);
     }
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract ProjectConfigurationActionContainer getConfigurationActions();
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract ModelRegistry getModelRegistry();
 
     @Override
@@ -1433,7 +1355,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract PluginManagerInternal getPluginManager();
 
     @Inject
@@ -1497,7 +1418,6 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public ExtensionContainerInternal getExtensions() {
-        onMutableStateAccess();
         return (ExtensionContainerInternal) extensibleDynamicObject.getExtensions();
     }
 
@@ -1523,31 +1443,26 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Override
     public void addDeferredConfiguration(Runnable configuration) {
-        // onMutableStateAccess() triggered by #getDeferredProjectConfiguration()
         getDeferredProjectConfiguration().add(configuration);
     }
 
     @Override
     public void fireDeferredConfiguration() {
-        // onMutableStateAccess() triggered by #getDeferredProjectConfiguration()
         getDeferredProjectConfiguration().fire();
     }
 
     @Override
     public void apply(Closure closure) {
-        onMutableStateAccess();
         super.apply(closure);
     }
 
     @Override
     public void apply(Action<? super ObjectConfigurationAction> action) {
-        onMutableStateAccess();
         super.apply(action);
     }
 
     @Override
     public void apply(Map<String, ?> options) {
-        onMutableStateAccess();
         super.apply(options);
     }
 
@@ -1570,23 +1485,19 @@ public abstract class DefaultProject extends AbstractPluginAware implements Proj
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract InputNormalizationHandlerInternal getNormalization();
 
     @Override
     public void normalization(Action<? super InputNormalizationHandler> configuration) {
-        //onMutableStateAccess(); triggered by #getNormalization()
         configuration.execute(getNormalization());
     }
 
     @Inject
     @Override
-    // onMutableStateAccess() triggered by #getServices()
     public abstract DependencyLockingHandler getDependencyLocking();
 
     @Override
     public void dependencyLocking(Action<? super DependencyLockingHandler> configuration) {
-        // onMutableStateAccess() triggered by #getDependencyLocking()
         configuration.execute(getDependencyLocking());
     }
 
