@@ -55,8 +55,8 @@ class HtmlDependencyVerificationReportRenderer implements DependencyVerification
     private final File verificationFile;
     private final List<String> writeFlags;
     private final File htmlReportOutputDirectory;
+    private final boolean useKeyServers;
     private boolean hasMissingKeys = false;
-    private boolean useKeyServers;
 
     HtmlDependencyVerificationReportRenderer(DocumentationRegistry documentationRegistry, File verificationFile, List<String> writeFlags, File htmlReportOutputDirectory, boolean useKeyServers) {
         this.documentationRegistry = documentationRegistry;
@@ -400,7 +400,11 @@ class HtmlDependencyVerificationReportRenderer implements DependencyVerification
                         reportItem(reason, "ignored-key", "info");
                         break;
                     case MISSING_KEY:
-                        reason = warning("Key " + keyInfo + " couldn't be found in any key server so verification couldn't be performed");
+                        if (useKeyServers) {
+                            reason = warning("Key " + keyInfo + " couldn't be found in local key file or remote key servers so verification couldn't be performed.");
+                        } else {
+                            reason = warning("Key " + keyInfo + " couldn't be found in local key file so verification couldn't be performed. Enable key resolution with --export-keys.");
+                        }
                         reportItem(reason, "missing-key", "warning");
                         hasMissingKeys = true;
                         break;
@@ -433,10 +437,6 @@ class HtmlDependencyVerificationReportRenderer implements DependencyVerification
             currentArtifact = artifactErrors;
         }
 
-        public String getTitle() {
-            return title;
-        }
-
     }
 
     private static class ArtifactErrors {
