@@ -24,6 +24,7 @@ import org.gradle.api.internal.artifacts.component.ComponentSelectorInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.specs.ExcludeSpec;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphEdge;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.strict.StrictVersionConstraints;
 import org.gradle.api.internal.attributes.AttributeMergingException;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.api.internal.attributes.immutable.ImmutableAttributesSchema;
@@ -35,6 +36,7 @@ import org.gradle.internal.component.model.VariantGraphResolveState;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
 import org.jspecify.annotations.Nullable;
 
+import java.util.Collection;
 import java.util.Collections;
 import java.util.LinkedList;
 import java.util.List;
@@ -56,7 +58,7 @@ class EdgeState implements DependencyGraphEdge {
     private final boolean isTransitive;
     private final boolean isConstraint;
 
-    private SelectorState selector;
+    private @Nullable SelectorState selector;
     private ModuleVersionResolveException targetNodeSelectionFailure;
 
     /**
@@ -68,7 +70,6 @@ class EdgeState implements DependencyGraphEdge {
 
     private @Nullable NodeState resolvedVariant;
     private boolean unattached;
-    private boolean used;
 
     EdgeState(NodeState from, DependencyState dependencyState, ResolveState resolveState) {
         this.from = from;
@@ -79,8 +80,24 @@ class EdgeState implements DependencyGraphEdge {
         this.isConstraint = dependencyMetadata.isConstraint();
     }
 
-    void computeSelector() {
-        this.selector = resolveState.computeSelectorFor(dependencyState, from.versionProvidedByAncestors(dependencyState));
+    boolean computeSelector(StrictVersionConstraints ancestorsStrictVersions, boolean deferSelection) {
+        boolean ignoreVersion = !dependencyState.isForced() && ancestorsStrictVersions.contains(dependencyState.getModuleIdentifier(resolveState.getComponentSelectorConverter()));
+        SelectorState newSelector = resolveState.computeSelectorFor(dependencyState, ignoreVersion);
+        if (this.selector != newSelector) {
+            clearSelector();
+            newSelector.use(deferSelection);
+            this.selector = newSelector;
+            return true;
+        }
+
+        return false;
+    }
+
+    public void clearSelector() {
+        if (this.selector != null) {
+            this.selector.release();
+            this.selector = null;
+        }
     }
 
     @Override
@@ -104,7 +121,7 @@ class EdgeState implements DependencyGraphEdge {
      */
     @Nullable
     ComponentState getTargetComponent() {
-        if (!selector.isResolved() || selector.getFailure() != null) {
+        if (selector == null || !selector.isResolved() || selector.getFailure() != null) {
             return null;
         }
         return getSelectedComponent();
@@ -112,6 +129,7 @@ class EdgeState implements DependencyGraphEdge {
 
     @Override
     public SelectorState getSelector() {
+        assert selector != null : "No selector for " + this;
         return selector;
     }
 
@@ -127,20 +145,12 @@ class EdgeState implements DependencyGraphEdge {
             return;
         }
 
-        if (isConstraint) {
-            // Need to double check that the target still has hard edges to it
-            ModuleResolveState module = targetComponent.getModule();
-            if (module.isPending()) {
-                selector.getTargetModule().removeUnattachedEdge(this);
-                from.makePending(this);
-                module.registerConstraintProvider(from);
-                return;
-            }
-        }
+        // We should never try to attach edges to a node in a module that has no incoming hard edges.
+        assert !targetComponent.getModule().isPending();
 
         calculateTargetNodes(targetComponent);
-        for (NodeState targetConfiguration : targetNodes) {
-            targetConfiguration.addIncomingEdge(this);
+        for (NodeState targetNode : targetNodes) {
+            targetNode.addIncomingEdge(this);
         }
         if (!targetNodes.isEmpty()) {
             selector.getTargetModule().removeUnattachedEdge(this);
@@ -199,7 +209,7 @@ class EdgeState implements DependencyGraphEdge {
 
     private void calculateTargetNodes(ComponentState targetComponent) {
         ComponentGraphResolveState targetComponentState = targetComponent.getResolveStateOrNull();
-        targetNodes.clear();
+        targetNodes.clear(); // TODO: Why not `detachFromTargetNodes()`?
         targetNodeSelectionFailure = null;
         if (targetComponentState == null) {
             targetComponent.getModule().getPlatformState().addOrphanEdge(this);
@@ -422,6 +432,10 @@ class EdgeState implements DependencyGraphEdge {
         }
     }
 
+    public Collection<NodeState> getTargetNodes() {
+        return targetNodes;
+    }
+
     @Nullable
     public NodeState getSelectedNode() {
         if (resolvedVariant != null) {
@@ -463,11 +477,6 @@ class EdgeState implements DependencyGraphEdge {
         return isConstraint;
     }
 
-    @Override
-    public long getFromVariant() {
-        return from.getNodeId();
-    }
-
     @Nullable
     private ComponentState getSelectedComponent() {
         return selector.getTargetModule().getSelected();
@@ -493,6 +502,17 @@ class EdgeState implements DependencyGraphEdge {
         }
     }
 
+    void recomputeSelectorAndRequeueTargetNodes(StrictVersionConstraints ancestorsStrictVersions, Collection<EdgeState> discoveredEdges) {
+        if (computeSelector(ancestorsStrictVersions, false)) {
+            discoveredEdges.add(this);
+        }
+        // TODO: If we compute the selector for this edge and it changes, we shouldn't add the (potentially) invalid target nodes to the queue.
+        // If we added this edge to `discoveredEdges`, then we will recompute target nodes and there is no point in adding the current target nodes to the queue.
+        for (NodeState targetNode : targetNodes) {
+            resolveState.onMoreSelected(targetNode);
+        }
+    }
+
     @Nullable
     ExcludeSpec getTransitiveExclusions() {
         return transitiveExclusions;
@@ -502,7 +522,7 @@ class EdgeState implements DependencyGraphEdge {
         this.unattached = true;
     }
 
-    public void markAttached() {
+    public void markNotUnattached() {
         this.unattached = false;
     }
 
@@ -510,14 +530,6 @@ class EdgeState implements DependencyGraphEdge {
         return unattached;
     }
 
-    void markUsed() {
-        this.used = true;
-    }
-
-    void markUnused() {
-        this.used = false;
-    }
-
     /**
      * Indicates whether the edge is currently listed as outgoing in a node.
      * It can be either a full edge or an edge to a virtual platform.
@@ -525,7 +537,7 @@ class EdgeState implements DependencyGraphEdge {
      * @return true if used, false otherwise
      */
     boolean isUsed() {
-        return used;
+        return selector != null;
     }
 
     public boolean isArtifactOnlyEdge() {
