@@ -17,12 +17,11 @@
 package org.gradle.api.internal.tasks.testing.junit.result;
 
 import com.google.common.annotations.VisibleForTesting;
-import org.gradle.api.Action;
 import org.gradle.api.GradleException;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
-import org.gradle.internal.FileUtils;
-import org.gradle.internal.IoActions;
+import org.gradle.internal.SafeFileLocationUtils;
+import org.gradle.internal.nativeintegration.network.HostnameLookup;
 import org.gradle.internal.operations.BuildOperationContext;
 import org.gradle.internal.operations.BuildOperationDescriptor;
 import org.gradle.internal.operations.BuildOperationExecutor;
@@ -32,11 +31,18 @@ import org.gradle.internal.operations.RunnableBuildOperation;
 import org.gradle.internal.time.Time;
 import org.gradle.internal.time.Timer;
 import org.gradle.util.internal.GFileUtils;
+import org.jspecify.annotations.NullMarked;
 
+import javax.inject.Inject;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.FilenameFilter;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
+@NullMarked
 public class Binary2JUnitXmlReportGenerator {
     private static final String REPORT_FILE_PREFIX = "TEST-";
     private static final String REPORT_FILE_EXTENSION = ".xml";
@@ -51,10 +57,14 @@ public class Binary2JUnitXmlReportGenerator {
     private final BuildOperationExecutor buildOperationExecutor;
     private final static Logger LOG = Logging.getLogger(Binary2JUnitXmlReportGenerator.class);
 
-    public Binary2JUnitXmlReportGenerator(File testResultsDir, TestResultsProvider testResultsProvider, JUnitXmlResultOptions options, BuildOperationRunner buildOperationRunner, BuildOperationExecutor buildOperationExecutor, String hostName) {
+    @Inject
+    public Binary2JUnitXmlReportGenerator(
+        BuildOperationRunner buildOperationRunner, BuildOperationExecutor buildOperationExecutor, HostnameLookup hostnameLookup,
+        File testResultsDir, TestResultsProvider testResultsProvider, JUnitXmlResultOptions options
+    ) {
         this.testResultsDir = testResultsDir;
         this.testResultsProvider = testResultsProvider;
-        this.xmlWriter = new JUnitXmlResultWriter(hostName, testResultsProvider, options);
+        this.xmlWriter = new JUnitXmlResultWriter(hostnameLookup.getHostname(), testResultsProvider, options);
         this.buildOperationRunner = buildOperationRunner;
         this.buildOperationExecutor = buildOperationExecutor;
     }
@@ -83,24 +93,42 @@ public class Binary2JUnitXmlReportGenerator {
             }
         });
 
-        buildOperationExecutor.runAll(new Action<BuildOperationQueue<JUnitXmlReportFileGenerator>>() {
-            @Override
-            public void execute(final BuildOperationQueue<JUnitXmlReportFileGenerator> queue) {
-                testResultsProvider.visitClasses(new Action<TestClassResult>() {
-                    @Override
-                    public void execute(final TestClassResult result) {
-                        final File reportFile = new File(testResultsDir, getReportFileName(result));
-                        queue.add(new JUnitXmlReportFileGenerator(result, reportFile, xmlWriter));
-                    }
-                });
+        // Collect all results first to detect duplicate class names
+        List<TestClassResult> allResults = new ArrayList<>();
+        testResultsProvider.visitClasses(allResults::add);
+
+        // Count occurrences of each class name
+        Map<String, Integer> classNameCounts = new HashMap<>();
+        for (TestClassResult result : allResults) {
+            classNameCounts.merge(result.getClassName(), 1, Integer::sum);
+        }
+
+        // Track current index for each duplicate class name
+        Map<String, Integer> classNameCurrentIndex = new HashMap<>();
+
+        buildOperationExecutor.runAll((BuildOperationQueue<JUnitXmlReportFileGenerator> queue) -> {
+            for (TestClassResult result : allResults) {
+                String className = result.getClassName();
+                String fileName;
+                if (classNameCounts.get(className) > 1) {
+                    // Duplicate class name - add numeric suffix
+                    int index = classNameCurrentIndex.merge(className, 1, Integer::sum);
+                    fileName = getReportFileName(className, index);
+                } else {
+                    // Unique class name - no suffix needed
+                    fileName = getReportFileName(className, 0);
+                }
+                File reportFile = new File(testResultsDir, fileName);
+                queue.add(new JUnitXmlReportFileGenerator(result, reportFile, xmlWriter));
             }
         });
 
         LOG.info("Finished generating test XML results ({}) into: {}", clock.getElapsed(), testResultsDir);
     }
 
-    private String getReportFileName(TestClassResult result) {
-        return REPORT_FILE_PREFIX + FileUtils.toSafeFileName(result.getClassName()) + REPORT_FILE_EXTENSION;
+    private static String getReportFileName(String className, int index) {
+        String suffix = index > 0 ? "-" + index : "";
+        return SafeFileLocationUtils.toSafeFileName(REPORT_FILE_PREFIX + className + suffix + REPORT_FILE_EXTENSION, false);
     }
 
     private static class JUnitXmlReportFileGenerator implements RunnableBuildOperation {
@@ -121,15 +149,10 @@ public class Binary2JUnitXmlReportGenerator {
 
         @Override
         public void run(BuildOperationContext context) {
-            FileOutputStream output = null;
-            try {
-                output = new FileOutputStream(reportFile);
+            try (FileOutputStream output = new FileOutputStream(reportFile)) {
                 xmlWriter.write(result, output);
-                output.close();
             } catch (Exception e) {
                 throw new GradleException(String.format("Could not write XML test results for %s to file %s.", result.getClassName(), reportFile), e);
-            } finally {
-                IoActions.closeQuietly(output);
             }
         }
     }
